import os
import requests
from tqdm import tqdm
from blueness import module
from abcli import file
from vancouver_watching import NAME
from vancouver_watching.logger import logger

NAME = module.name(__file__, NAME)


def discover_cameras_vancouver_style(filename, prefix):
    from vancouver_watching.QGIS import label_of_camera
    from bs4 import BeautifulSoup

    logger.info(f"{NAME}.discover_cameras({filename}): vancouver-style")

    success, gdf = file.load_geodataframe(filename)
    if not success:
        return False

    list_of_cameras = []
    list_of_labels = []
    failed_locations = []
    for _, row in tqdm(gdf.iterrows()):
        list_of_cameras_ = []

        try:
            html_page = requests.get(row["url"])

            # https://towardsdatascience.com/a-tutorial-on-scraping-images-from-the-web-using-beautifulsoup-206a7633e948
            soup = BeautifulSoup(html_page.content, "html.parser")
            list_of_cameras_ = [
                f'{prefix}{item.attrs["src"]}'
                for item in soup.find(
                    "div",
                    class_="col-sm-12 section--container",
                ).findAll("img")
            ]
        except:
            failed_locations += [row["url"]]
            logger.error(f"failed: {row['url']}")

        list_of_cameras += [",".join(list_of_cameras_)]
        list_of_labels += [
            label_of_camera(
                row["url"],
                row["name"],
                list_of_cameras_,
            )
        ]
    if failed_locations:
        logger.error(f"{len(failed_locations)} location(s) failed.")
    logger.info(
        "found {} camera(s) in {} location(s).".format(
            len(
                [camera for camera in (",".join(list_of_cameras)).split(",") if camera]
            ),
            len(gdf),
        )
    )

    gdf["cameras"] = list_of_cameras
    gdf["label"] = list_of_labels

    return file.save_geojson(filename, gdf)


def get_list_of_areas():
    return [
        file.name(filename)
        for filename in file.list_of(
            os.path.join(
                file.path(__file__),
                "../.abcli/discovery/*.sh",
            )
        )
    ]



import argparse
from blueness import module
from vancouver_watching import NAME, VERSION
from vancouver_watching.discover.functions import (
    discover_cameras_vancouver_style,
    get_list_of_areas,
)
from vancouver_watching.logger import logger
from blueness.argparse.generic import sys_exit


NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "task",
    type=str,
    help="discover_cameras_vancouver_style|list_of_areas",
)
parser.add_argument(
    "--filename",
    type=str,
    default="",
)
parser.add_argument(
    "--prefix",
    type=str,
    default="",
)
parser.add_argument(
    "--delim",
    type=str,
    default="+",
)
args = parser.parse_args()

delim = " " if args.delim == "space" else args.delim

success = False
if args.task == "list_of_areas":
    print(delim.join(get_list_of_areas()))
    success = True
elif args.task == "discover_cameras_vancouver_style":
    success = discover_cameras_vancouver_style(
        args.filename,
        args.prefix,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



import argparse
from blueness import module
from vancouver_watching import NAME, VERSION
from vancouver_watching.area import Area
from vancouver_watching.logger import logger
from blueness.argparse.generic import sys_exit

NAME = module.name(__file__, NAME)


parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "task",
    type=str,
    help="process",
)
parser.add_argument(
    "--geojson",
    type=str,
    default="",
)
parser.add_argument(
    "--count",
    type=int,
    default=-1,
    help="-1: all",
)
parser.add_argument(
    "--do_dryrun",
    type=int,
    default=0,
    help="0|1",
)
parser.add_argument(
    "--verbose",
    type=int,
    default=0,
    help="0|1",
)
parser.add_argument(
    "--overwrite",
    type=int,
    default=0,
    help="0|1",
)
parser.add_argument(
    "--animated_gif",
    type=int,
    default=0,
    help="0|1",
)
parser.add_argument(
    "--detect_objects",
    type=int,
    default=1,
    help="0|1",
)
parser.add_argument(
    "--model_id",
    type=str,
    default="",
)
args = parser.parse_args()

success = False
if args.task == "process":
    area = Area(
        args.geojson,
        do_dryrun=args.do_dryrun,
        verbose=args.verbose,
    )
    success = area.valid

    if success and args.detect_objects:
        success = area.detect_objects(
            model_id=args.model_id,
            animated_gif=args.animated_gif,
            count=args.count,
            overwrite=args.overwrite,
        )

    if success:
        success = area.summarize()
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from vancouver_watching import NAME, VERSION, DESCRIPTION, ICON
from vancouver_watching.logger import logger
from blueness.argparse.generic import main

success, message = main(
    __file__,
    NAME,
    VERSION,
    DESCRIPTION,
    ICON,
)
if not success:
    logger.error(message)



import argparse
from blueness import module
from vancouver_watching import NAME, VERSION
from vancouver_watching.logger import logger
from vancouver_watching.QGIS import update_cache
from blueness.argparse.generic import sys_exit

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "task",
    type=str,
    help="update",
)
parser.add_argument(
    "--object_name",
    type=str,
    default=".",
)
parser.add_argument(
    "--verbose",
    type=int,
    default=0,
    help="0|1",
)
args = parser.parse_args()

success = False
if args.task == "update":
    success, _ = update_cache(
        object_name=args.object_name,
        verbose=args.verbose,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



import os
import os.path
from typing import List
import re
from collections import Counter
from tqdm import tqdm
from abcli import file, path
from abcli.plugins.graphics.gif import generate_animated_gif
from vancouver_watching.ai.classes import Ultralytics_API
from vancouver_watching.logger import logger


class Area:
    def __init__(
        self,
        map_filename: str,
        do_dryrun: bool = False,
        verbose: bool = False,
    ):
        import geopandas

        self.map_filename = map_filename  # geojson file

        self.object_path = file.path(self.map_filename)
        self.object_name = path.name(self.object_path)
        self.gdf = geopandas.read_file(self.map_filename)

        self.do_dryrun = do_dryrun
        self.verbose = verbose

        self.metadata_filename = file.set_extension(self.map_filename, "json")
        self.metadata = {}

        self.valid = True
        for column in "cameras,mapid".split(","):
            if column not in self.gdf.columns:
                self.valid = False
                logger.info(
                    "Invaid Area, missing {}: {}.".format(column, self.map_filename)
                )
                return

        logger.info(
            "{}: {} mapid(s) from {}.".format(
                self.__class__.__name__,
                len(self.gdf),
                self.map_filename,
            )
        )

        success, self.metadata = file.load_json(self.metadata_filename, civilized=True)
        if not success:
            logger.info("generating metadata: {}".format(self.metadata_filename))

            p = re.compile(
                "https?:\/\/([0-9.]+).\:([0-9.]+)\/webcapture.jpg.*.channel=([0-9.]+).*"
            )
            for _, row in tqdm(self.gdf.iterrows()):
                cameras = {}
                for url in row["cameras"].split(","):
                    matches = p.match(url)
                    if matches:
                        filename = (
                            f"{matches[1]}-{matches[2]}-{matches[3]}".replace(".", "-")
                            + ".jpg"
                        )
                    else:
                        filename = file.name_and_extension(url)

                    if file.extension(filename) not in "jpg,jpeg,png".split(","):
                        logger.error("bad url: {}.".format(url))
                        continue

                    cameras[filename] = {"url": url}

                self.metadata[row["mapid"]] = {"cameras": cameras}

            self.save_metadata()

    def detect_objects(
        self,
        model_id: str,
        animated_gif: bool = False,
        count: int = -1,
        overwrite: bool = False,
    ) -> bool:
        logger.info(
            "{}.detect_objects({},model_id{},count={}{})".format(
                self.__class__.__name__,
                model_id,
                ",animated_gif" if animated_gif else "",
                count,
                ",overwrite" if overwrite else "",
            )
        )

        ultralytics_api = Ultralytics_API(
            model_id,
            self.do_dryrun,
            self.verbose,
        )

        list_of_images: List[str] = []
        counter: int = 0
        for mapid in tqdm(self.metadata):
            for filename, metadata in self.metadata[mapid]["cameras"].items():
                full_filename = os.path.join(self.object_path, filename)
                if not file.exist(full_filename):
                    continue

                if self.do_dryrun:
                    logger.info(full_filename)
                    continue

                if not overwrite and "inference" in metadata:
                    continue

                success, inference = ultralytics_api.infer(full_filename)
                if success:
                    metadata["inference"] = inference
                    list_of_images += [inference.get("render_filename", "")]

                    counter += 1
                    if count != -1 and counter >= count:
                        break
            if count != -1 and counter >= count:
                break

        if not self.save_metadata():
            return False

        return not animated_gif or generate_animated_gif(
            [filename for filename in list_of_images if filename],
            os.path.join(self.object_path, f"{self.object_name}.gif"),
            frame_duration=500,
        )

    def ingest(
        self,
        count: int,
    ) -> bool:
        logger.info("{}.ingest({})".format(self.__class__.__name__, count))
        counter = 0
        for mapid in tqdm(self.metadata):
            for filename, metadata in self.metadata[mapid]["cameras"].items():
                url = metadata["url"]

                if self.do_dryrun:
                    logger.info(url)
                elif not file.download(url, os.path.join(self.object_path, filename)):
                    logger.error("bad url: {}.".format(url))

                counter += 1
                if counter >= count and count != -1:
                    break
            if counter >= count and count != -1:
                break

        return True

    def on_map(
        self,
        zoom_start: int = 12,
    ):
        import folium

        if self.gdf.empty:
            return None

        output = folium.Map(
            location=self.gdf.unary_union.centroid.coords[0][::-1],
            zoom_start=zoom_start,
        )
        folium.GeoJson(self.gdf).add_to(output)

        return output

    def save_gdf(self) -> bool:
        return file.save_geojson(self.map_filename, self.gdf, log=True)

    def save_metadata(self) -> bool:
        return file.save_json(self.metadata_filename, self.metadata, log=True)

    def summarize(self) -> bool:
        all_things = {}
        for mapid in tqdm(self.metadata):
            detections = {}
            for metadata in self.metadata[mapid]["cameras"].values():
                if not "inference" in metadata:
                    continue

                for thing, count in Counter(
                    [thing["name"] for thing in metadata["inference"].get("data", {})]
                ).items():
                    detections[thing] = detections.get(thing, 0) + count

            for thing, count in detections.items():
                all_things[thing] = all_things.get(thing, 0) + count

                if thing not in self.gdf.columns:
                    self.gdf[thing] = 0
                    logger.info("+= {}".format(thing))

                self.gdf.loc[self.gdf["mapid"] == mapid, thing] += count

        logger.info(
            ", ".join(
                ["{}: {}".format(thing, count) for thing, count in all_things.items()]
            )
        )

        return self.save_gdf()



import argparse
from blueness import module
from vancouver_watching import NAME, VERSION
from vancouver_watching.area import Area
from vancouver_watching.logger import logger
from blueness.argparse.generic import sys_exit

NAME = module.name(__file__, NAME)


parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "--geojson",
    type=str,
    default="",
)
parser.add_argument(
    "--count",
    type=int,
    default=-1,
)
parser.add_argument(
    "--do_dryrun",
    type=int,
    default=0,
    help="0|1",
)
args = parser.parse_args()

area = Area(
    args.geojson,
    do_dryrun=args.do_dryrun,
)
success = area.valid

if success:
    success = area.ingest(count=args.count)


sys_exit(logger, NAME, "discover", success)
